@@ -5,10 +5,14 @@ use thiserror::Error;
 /// Describes errors in the initialization of `flexi_logger`.
 #[derive(Error, Debug)]
 pub enum FlexiLoggerError {
-    /// Reset not possible because not file logger is configured.
-    #[error("Reset not possible because not file logger is configured")]
+    /// Chosen reset not possible.
+    #[error("Chosen reset not possible")]
     Reset,
 
+    /// Method not possible because no file logger is configured.
+    #[error("Method not possible because no file logger is configured")]
+    NoFileLogger,
+
     /// Log file cannot be written because the specified path is not a directory.
     #[error("Log file cannot be written because the specified path is not a directory")]
     OutputBadDirectory,
@@ -30,9 +34,9 @@ pub enum FlexiLoggerError {
     OutputIo(#[from] std::io::Error),
 
     /// Filesystem notifications for the specfile could not be set up.
-    #[error("Filesystem notifications for the specfile could not be set up")]
-    #[cfg(feature = "specfile")]
-    SpecfileNotify(#[from] notify::Error),
+    #[error("Filesystem notifications for the specfile or the log file could not be set up")]
+    #[cfg(feature = "notify")]
+    Notify(#[from] notify::Error),
 
     /// Parsing the configured logspec toml-file failed.
     #[error("Parsing the configured logspec toml-file failed")]
