@@ -3,14 +3,13 @@ use crate::DeferredNow;
 use ansi_term::{Color, Style};
 use log::Record;
 use std::thread;
-use time::format_description;
+use time::{format_description::FormatItem, macros::format_description};
 
-// const TS_S: &str = "[year]-[month]-[day] [hour]:[minute]:[second].[subsecond digits:6] \
-// [offset_hour sign:mandatory]:[offset_minute]";
-// lazy_static::lazy_static! {
-//     static ref TS: Vec<format_description::FormatItem<'static>>
-//         = format_description::parse(TS_S).unwrap(/*ok*/);
-// }
+/// Precompiled version of the time stamp format that is used by the provided format functions.
+pub const TS_DASHES_BLANK_COLONS_DOT_BLANK: &[FormatItem<'static>] = format_description!(
+    "[year]-[month]-[day] [hour]:[minute]:[second].[subsecond digits:6] \
+        [offset_hour sign:mandatory]:[offset_minute]"
+);
 
 /// A logline-formatter that produces log lines like <br>
 /// ```INFO [my_prog::some_submodule] Task successfully read from conf.json```
@@ -60,13 +59,6 @@ pub fn colored_default_format(
     )
 }
 
-const TS_S: &str = "[year]-[month]-[day] [hour]:[minute]:[second].[subsecond digits:6] \
-                    [offset_hour sign:mandatory]:[offset_minute]";
-lazy_static::lazy_static! {
-    static ref TS: Vec<format_description::FormatItem<'static>>
-        = format_description::parse(TS_S).unwrap(/*ok*/);
-}
-
 /// A logline-formatter that produces log lines with timestamp and file location, like
 /// <br>
 /// ```[2016-01-13 15:25:01.640870 +01:00] INFO [src/foo/bar:26] Task successfully read from conf.json```
@@ -83,9 +75,7 @@ pub fn opt_format(
     write!(
         w,
         "[{}] {} [{}:{}] {}",
-        now.now()
-            .format(&TS)
-            .unwrap_or_else(|_| "Timestamping failed".to_string()),
+        now.format(TS_DASHES_BLANK_COLONS_DOT_BLANK),
         record.level(),
         record.file().unwrap_or("<unnamed>"),
         record.line().unwrap_or(0),
@@ -111,11 +101,7 @@ pub fn colored_opt_format(
     write!(
         w,
         "[{}] {} [{}:{}] {}",
-        style(level).paint(
-            now.now()
-                .format(&TS)
-                .unwrap_or_else(|_| "Timestamping failed".to_string())
-        ),
+        style(level).paint(now.format(TS_DASHES_BLANK_COLONS_DOT_BLANK)),
         style(level).paint(level.to_string()),
         record.file().unwrap_or("<unnamed>"),
         record.line().unwrap_or(0),
@@ -140,9 +126,7 @@ pub fn detailed_format(
     write!(
         w,
         "[{}] {} [{}] {}:{}: {}",
-        now.now()
-            .format(&TS)
-            .unwrap_or_else(|_| "Timestamping failed".to_string()),
+        now.format(TS_DASHES_BLANK_COLONS_DOT_BLANK),
         record.level(),
         record.module_path().unwrap_or("<unnamed>"),
         record.file().unwrap_or("<unnamed>"),
@@ -169,11 +153,7 @@ pub fn colored_detailed_format(
     write!(
         w,
         "[{}] {} [{}] {}:{}: {}",
-        style(level).paint(
-            now.now()
-                .format(&TS)
-                .unwrap_or_else(|_| "Timestamping failed".to_string())
-        ),
+        style(level).paint(now.format(TS_DASHES_BLANK_COLONS_DOT_BLANK)),
         style(level).paint(record.level().to_string()),
         record.module_path().unwrap_or("<unnamed>"),
         record.file().unwrap_or("<unnamed>"),
@@ -199,9 +179,7 @@ pub fn with_thread(
     write!(
         w,
         "[{}] T[{:?}] {} [{}:{}] {}",
-        now.now()
-            .format(&TS)
-            .unwrap_or_else(|_| "Timestamping failed".to_string()),
+        now.format(TS_DASHES_BLANK_COLONS_DOT_BLANK),
         thread::current().name().unwrap_or("<unnamed>"),
         record.level(),
         record.file().unwrap_or("<unnamed>"),
@@ -228,11 +206,7 @@ pub fn colored_with_thread(
     write!(
         w,
         "[{}] T[{:?}] {} [{}:{}] {}",
-        style(level).paint(
-            now.now()
-                .format(&TS)
-                .unwrap_or_else(|_| "Timestamping failed".to_string())
-        ),
+        style(level).paint(now.format(TS_DASHES_BLANK_COLONS_DOT_BLANK)),
         style(level).paint(thread::current().name().unwrap_or("<unnamed>")),
         style(level).paint(level.to_string()),
         record.file().unwrap_or("<unnamed>"),
