@@ -31,24 +31,10 @@ use std::time::Duration;
 
 /// The entry-point for using `flexi_logger`.
 ///
-/// A simple example with file logging might look like this:
-///
-/// ```rust
-/// use flexi_logger::{Duplicate, FileSpec, Logger};
-///
-/// Logger::try_with_str("info, mycrate = debug")
-///         .unwrap()
-///         .log_to_file(FileSpec::default())
-///         .duplicate_to_stderr(Duplicate::Warn)
-///         .start()
-///         .unwrap();
-/// ```
-///
-///
 /// `Logger` is a builder class that allows you to
 /// * specify your desired (initial) loglevel-specification
-///   * either programmatically as a String ([`Logger::try_with_str`])
-///   * or by providing a String in the environment ([`Logger::try_with_env`]),
+///   * either as a String ([`Logger::try_with_str`])
+///   * or by providing it in the environment ([`Logger::try_with_env`]),
 ///   * or by combining both options ([`Logger::try_with_env_or_str`]),
 ///   * or by building a [`LogSpecification`] programmatically ([`Logger::with`]),
 /// * use the desired configuration methods,
@@ -56,6 +42,10 @@ use std::time::Duration;
 ///
 ///   * [`Logger::start`], or
 ///   * [`Logger::start_with_specfile`].
+///
+/// # Usage
+///
+/// See [`code_examples`](code_examples/index.html) for a comprehensive list of usage possibilities.
 pub struct Logger {
     spec: LogSpecification,
     log_target: LogTarget,
@@ -105,9 +95,11 @@ impl Logger {
     /// Creates a Logger that reads the [`LogSpecification`] from the environment variable
     /// `RUST_LOG`.
     ///
+    /// Note that if `RUST_LOG` is not set, nothing is logged.
+    ///
     /// # Errors
     ///
-    /// `FlexiLoggerError::Parse` if the value of `RUST_LOG` uses an erroneous syntax.
+    /// `FlexiLoggerError::Parse` if the value of `RUST_LOG` is malformed.
     pub fn try_with_env() -> Result<Self, FlexiLoggerError> {
         Ok(Self::from_spec_and_errs(LogSpecification::env()?))
     }
@@ -117,7 +109,7 @@ impl Logger {
     ///
     /// # Errors
     ///
-    /// `FlexiLoggerError::Parse` if the used String uses an erroneous syntax.
+    /// `FlexiLoggerError::Parse` if the chosen value is malformed.
     pub fn try_with_env_or_str<S: AsRef<str>>(s: S) -> Result<Self, FlexiLoggerError> {
         Ok(Self::from_spec_and_errs(LogSpecification::env_or_parse(s)?))
     }
@@ -277,6 +269,7 @@ impl Logger {
     /// By default, [`default_format`] is used for output to files and to custom writers,
     /// and [`AdaptiveFormat::Default`] is used for output to `stderr` and `stdout`.
     /// If the feature `colors` is switched off, [`default_format`] is used for all outputs.
+    #[must_use]
     pub fn format(mut self, format: FormatFunction) -> Self {
         self.format_for_file = format;
         self.format_for_stderr = format;
@@ -289,6 +282,7 @@ impl Logger {
     /// that are written to files.
     ///
     /// Regarding the default, see [`Logger::format`].
+    #[must_use]
     pub fn format_for_files(mut self, format: FormatFunction) -> Self {
         self.format_for_file = format;
         self
@@ -298,6 +292,7 @@ impl Logger {
     /// that are written to stderr.
     ///
     /// Regarding the default, see [`Logger::format`].
+    #[must_use]
     pub fn format_for_stderr(mut self, format_function: FormatFunction) -> Self {
         self.format_for_stderr = format_function;
         self
@@ -319,6 +314,7 @@ impl Logger {
     /// that are written to stdout.
     ///
     /// Regarding the default, see [`Logger::format`].
+    #[must_use]
     pub fn format_for_stdout(mut self, format_function: FormatFunction) -> Self {
         self.format_for_stdout = format_function;
         self
@@ -341,6 +337,7 @@ impl Logger {
     /// whether it evaluates this setting or not.
     ///
     /// Regarding the default, see [`Logger::format`].
+    #[must_use]
     pub fn format_for_writer(mut self, format: FormatFunction) -> Self {
         self.format_for_writer = format;
         self
@@ -457,6 +454,27 @@ impl Logger {
         self
     }
 
+    /// Makes the logger react cooperatively if external tools rename or delete the log file.
+    ///
+    /// By default, `flexi_logger` decides, based on your configuration, to which destination(s)
+    /// the log is written.
+    ///
+    /// If logs are written to files, `flexi_logger` expects that nobody else modifies these,
+    /// and it offers capabilities to rotate, compress, and clean up log files.
+    ///
+    /// Alternatively, tools like linux' `logrotate` can be used to rotate, compress or remove
+    /// log files. But renaming or deleting the current output file e.g. will not stop
+    /// `flexi_logger` from writing to the now renamed or even deleted file!
+    /// You should configure `flexi_logger` with `Logger::watch_external_rotations()`
+    /// to make it watch for OS events that affect its outputfile
+    /// and react with closing its current output stream and recreating its configured output file.
+    #[must_use]
+    #[cfg(feature = "external_rotation")]
+    pub fn watch_external_rotations(mut self) -> Self {
+        self.flwb = self.flwb.watch_external_rotations();
+        self
+    }
+
     /// Makes the logger use UTC timestamps rather than local timestamps.
     #[must_use]
     pub fn use_utc(mut self) -> Self {
@@ -464,7 +482,7 @@ impl Logger {
         self
     }
 
-    /// The specified path will be used on linux systems to create a symbolic link
+    /// The specified path will be used on unix systems to create a symbolic link
     /// to the current log file.
     ///
     /// This option has no effect on filesystems where symlinks are not supported,
@@ -481,6 +499,7 @@ impl Logger {
     /// tail --follow=name --max-unchanged-stats=1 --retry link_to_log_file
     /// ```
     ///
+    #[must_use]
     pub fn create_symlink<P: Into<PathBuf>>(mut self, symlink: P) -> Self {
         self.flwb = self.flwb.create_symlink(symlink);
         self
@@ -490,6 +509,7 @@ impl Logger {
     ///
     /// The target name must not start with an underscore.
     /// See module [`writers`](crate::writers) for more details.
+    #[must_use]
     pub fn add_writer<S: Into<String>>(
         mut self,
         target_name: S,
@@ -580,22 +600,22 @@ impl Logger {
         self
     }
 
-    /// This option only has an effect if `log_to_file` is set to true.
-    ///
     /// If append is set to true, makes the logger append to the specified output file, if it exists.
     /// By default, or with false, the file would be truncated.
     ///
-    /// This option will hardly make an effect if `suppress_timestamp()` is not used.
+    /// This option only has an effect if logs are written to files,
+    /// and it will hardly make an effect if `suppress_timestamp()` is not used.
     #[must_use]
     pub fn o_append(mut self, append: bool) -> Self {
         self.flwb = self.flwb.o_append(append);
         self
     }
 
-    /// This option only has an effect if `log_to_file` is set to true.
-    ///
-    /// If a String is specified, it will be used on linux systems to create in the current folder
+    /// If a String is specified, it will be used on unix systems to create in the current folder
     /// a symbolic link with this name to the current log file.
+    ///
+    /// This option only has an effect on unix systems and if logs are written to files.
+    #[must_use]
     pub fn o_create_symlink<P: Into<PathBuf>>(mut self, symlink: Option<P>) -> Self {
         self.flwb = self.flwb.o_create_symlink(symlink);
         self
