@@ -1,11 +1,7 @@
 use crate::{writers::log_writer::LogWriter, DeferredNow};
-use std::cell::RefCell;
-use std::ffi::OsString;
-use std::io::Error as IoError;
-use std::io::Result as IoResult;
-use std::io::{BufWriter, ErrorKind, Write};
+use std::io::{Error as IoError, ErrorKind, Result as IoResult, Write};
 use std::net::{TcpStream, ToSocketAddrs, UdpSocket};
-#[cfg(target_os = "linux")]
+#[cfg(target_family = "unix")]
 use std::path::Path;
 use std::sync::Mutex;
 
@@ -64,7 +60,7 @@ pub enum SyslogFacility {
     LocalUse7 = 23 << 3,
 }
 
-/// [`SyslogConnector`]'s severity.
+/// Syslog severity.
 ///
 /// See [RFC 5424](https://datatracker.ietf.org/doc/rfc5424).
 #[derive(Debug)]
@@ -100,24 +96,20 @@ fn default_mapping(level: log::Level) -> SyslogSeverity {
     }
 }
 
-/// An experimental configurable [`LogWriter`] implementation that writes log messages to the syslog
+/// A configurable [`LogWriter`] implementation that writes log messages to the syslog
 /// (see [RFC 5424](https://datatracker.ietf.org/doc/rfc5424)).
 ///
 /// Only available with optional crate feature `syslog_writer`.
 ///
-/// For using the `SyslogWriter`, you need to know how the syslog is managed on your system,  
-/// how you can access it and with which protocol you can write to it,
-/// so that you can choose a variant of the `SyslogConnector` that fits to your environment.
-///
 /// See the [writers](crate::writers) for guidance how to use additional log writers.
 pub struct SyslogWriter {
-    hostname: OsString,
+    hostname: String,
     process: String,
     pid: u32,
     facility: SyslogFacility,
     message_id: String,
     determine_severity: LevelToSyslogSeverity,
-    syslog: Mutex<RefCell<SyslogConnector>>,
+    m_conn_buf: Mutex<ConnectorAndBuffer>,
     max_log_level: log::LevelFilter,
 }
 impl SyslogWriter {
@@ -136,7 +128,7 @@ impl SyslogWriter {
     /// is a string without further semantics. It is intended for filtering
     /// messages on a relay or collector.
     ///
-    /// `syslog`: A [`SyslogConnector`](crate::writers::SyslogConnector).
+    /// `syslog`: A [`Syslog`](crate::writers::Syslog).
     ///
     /// # Errors
     ///
@@ -146,13 +138,32 @@ impl SyslogWriter {
         determine_severity: Option<LevelToSyslogSeverity>,
         max_log_level: log::LevelFilter,
         message_id: String,
-        syslog: SyslogConnector,
+        syslog: Syslog,
     ) -> IoResult<Box<Self>> {
+        const UNKNOWN_HOSTNAME: &str = "<unknown_hostname>";
+
+        let hostname = hostname::get().map_or_else(
+            |_| Ok(UNKNOWN_HOSTNAME.to_owned()),
+            |s| {
+                s.into_string().map_err(|_| {
+                    IoError::new(
+                        ErrorKind::InvalidData,
+                        "Hostname contains non-UTF8 characters".to_owned(),
+                    )
+                })
+            },
+        )?;
+
+        let process = std::env::args().next().ok_or_else(|| {
+            IoError::new(
+                ErrorKind::Other,
+                "Can't infer app name as no env args are present".to_owned(),
+            )
+        })?;
+
         Ok(Box::new(Self {
-            hostname: hostname::get().unwrap_or_else(|_| OsString::from("<unknown_hostname>")),
-            process: std::env::args()
-                .next()
-                .ok_or_else(|| IoError::new(ErrorKind::Other, "<no progname>".to_owned()))?,
+            hostname,
+            process,
             pid: std::process::id(),
             facility,
             max_log_level,
@@ -163,35 +174,48 @@ impl SyslogWriter {
                 Some(f) => f,
                 None => default_mapping,
             },
-            syslog: Mutex::new(RefCell::new(syslog)),
+            m_conn_buf: Mutex::new(ConnectorAndBuffer {
+                conn: syslog.into_inner(),
+                buf: Vec::with_capacity(200),
+            }),
         }))
     }
 }
 
 impl LogWriter for SyslogWriter {
     fn write(&self, now: &mut DeferredNow, record: &log::Record) -> IoResult<()> {
-        let mr_syslog = self.syslog.lock().unwrap();
-        let mut syslog = mr_syslog.borrow_mut();
-
+        let mut conn_buf_guard = self
+            .m_conn_buf
+            .lock()
+            .map_err(|_| crate::util::io_err("SyslogWriter is poisoned"))?;
+        let cb = &mut *conn_buf_guard;
         let severity = (self.determine_severity)(record.level());
-        writeln!(
-            syslog,
-            "<{}>1 {} {:?} {} {} {} - {}",
-            self.facility as u8 | severity as u8,
-            now.format_rfc3339(),
-            self.hostname,
-            self.process,
-            self.pid,
-            self.message_id,
-            &record.args()
-        )
+
+        // See [RFC 5424](https://datatracker.ietf.org/doc/rfc5424#page-8).
+        cb.buf.clear();
+        #[allow(clippy::write_literal)]
+        write!(
+            cb.buf,
+            "<{pri}>{version} {timestamp} {hostname} {appname} {procid} {msgid} - {msg}",
+            pri = self.facility as u8 | severity as u8,
+            version = "1",
+            timestamp = now.format_rfc3339(),
+            hostname = self.hostname,
+            appname = self.process,
+            procid = self.pid,
+            msgid = self.message_id,
+            msg = &record.args()
+        )?;
+        // we _have_ to buffer because each write here generates a syslog entry
+        cb.conn.write_all(&cb.buf)
     }
 
     fn flush(&self) -> IoResult<()> {
-        let mr_syslog = self.syslog.lock().unwrap();
-        let mut syslog = mr_syslog.borrow_mut();
-        syslog.flush()?;
-        Ok(())
+        self.m_conn_buf
+            .lock()
+            .map_err(|_| crate::util::io_err("SyslogWriter is poisoned"))?
+            .conn
+            .flush()
     }
 
     fn max_log_level(&self) -> log::LevelFilter {
@@ -199,76 +223,57 @@ impl LogWriter for SyslogWriter {
     }
 }
 
-/// Helper struct that connects to the syslog and implements Write.
-///
-/// Is used in [`SyslogWriter::try_new`](crate::writers::SyslogWriter::try_new).
-///
-/// ## Example
-///
-/// ```rust,no_run
-///     use flexi_logger::writers::SyslogConnector;
-///    let syslog_connector = SyslogConnector::try_tcp("localhost:7777").unwrap();
-/// ```
-///
-#[derive(Debug)]
-pub enum SyslogConnector {
-    /// Sends log lines to the syslog via a
-    /// [UnixStream](https://doc.rust-lang.org/std/os/unix/net/struct.UnixStream.html).
-    #[cfg_attr(docsrs, doc(cfg(target_os = "linux")))]
-    #[cfg(target_os = "linux")]
-    Stream(BufWriter<std::os::unix::net::UnixStream>),
-
-    /// Sends log lines to the syslog via a
-    /// [UnixDatagram](https://doc.rust-lang.org/std/os/unix/net/struct.UnixDatagram.html).
-    #[cfg_attr(docsrs, doc(cfg(target_os = "linux")))]
-    #[cfg(target_os = "linux")]
-    Datagram(std::os::unix::net::UnixDatagram),
-
-    /// Sends log lines to the syslog via UDP.
-    ///
-    /// UDP is fragile and thus discouraged except for local communication.
-    Udp(UdpSocket),
-
-    /// Sends log lines to the syslog via TCP.
-    Tcp(BufWriter<TcpStream>),
+struct ConnectorAndBuffer {
+    conn: SyslogConnector,
+    buf: Vec<u8>,
 }
-impl SyslogConnector {
-    /// Returns a [`SyslogConnector::Datagram`] to the specified path.
+
+/// Implements the connection to the syslog.
+///
+/// Choose one of the factory methods that matches your environment,
+/// depending on how the syslog is managed on your system,  
+/// how you can access it and with which protocol you can write to it.
+///
+/// Is required to instantiate a [`SyslogWriter`](crate::writers::SyslogWriter).
+pub struct Syslog(SyslogConnector);
+impl Syslog {
+    /// Returns a Syslog implementation that connects via unix datagram to the specified path.
     ///
     /// # Errors
     ///
     /// Any kind of I/O error can occur.
-    #[cfg_attr(docsrs, doc(cfg(target_os = "linux")))]
-    #[cfg(target_os = "linux")]
-    pub fn try_datagram<P: AsRef<Path>>(path: P) -> IoResult<SyslogConnector> {
+    #[cfg_attr(docsrs, doc(cfg(target_family = "unix")))]
+    #[cfg(target_family = "unix")]
+    pub fn try_datagram<P: AsRef<Path>>(path: P) -> IoResult<Self> {
         let ud = std::os::unix::net::UnixDatagram::unbound()?;
         ud.connect(&path)?;
-        Ok(SyslogConnector::Datagram(ud))
+        Ok(Syslog(SyslogConnector::Datagram(ud)))
     }
 
-    /// Returns a [`SyslogConnector::Stream`] to the specified path.
+    /// Returns a Syslog implementation that connects via unix stream to the specified path.
     ///
     /// # Errors
     ///
     /// Any kind of I/O error can occur.
-    #[cfg_attr(docsrs, doc(cfg(target_os = "linux")))]
-    #[cfg(target_os = "linux")]
-    pub fn try_stream<P: AsRef<Path>>(path: P) -> IoResult<SyslogConnector> {
-        Ok(SyslogConnector::Stream(BufWriter::new(
+    #[cfg_attr(docsrs, doc(cfg(target_family = "unix")))]
+    #[cfg(target_family = "unix")]
+    pub fn try_stream<P: AsRef<Path>>(path: P) -> IoResult<Self> {
+        Ok(Syslog(SyslogConnector::Stream(
             std::os::unix::net::UnixStream::connect(path)?,
         )))
     }
 
-    /// Returns a [`SyslogConnector`] which sends the log lines via TCP to the specified address.
+    /// Returns a Syslog implementation that sends the log lines via TCP to the specified address.
     ///
     /// # Errors
     ///
     /// `std::io::Error` if opening the stream fails.
     pub fn try_tcp<T: ToSocketAddrs>(server: T) -> IoResult<Self> {
-        Ok(Self::Tcp(BufWriter::new(TcpStream::connect(server)?)))
+        Ok(Syslog(SyslogConnector::Tcp(TcpStream::connect(server)?)))
     }
 
-    /// Returns a `SyslogConnector` which sends log via the fragile UDP protocol from local to server.
+    /// Returns a Syslog implementation that sends the log via the fragile UDP protocol from local
+    /// to server.
     ///
     /// # Errors
     ///
@@ -276,37 +281,58 @@ impl SyslogConnector {
     pub fn try_udp<T: ToSocketAddrs>(local: T, server: T) -> IoResult<Self> {
         let socket = UdpSocket::bind(local)?;
         socket.connect(server)?;
-        Ok(Self::Udp(socket))
+        Ok(Syslog(SyslogConnector::Udp(socket)))
+    }
+
+    fn into_inner(self) -> SyslogConnector {
+        self.0
     }
 }
 
-impl Write for SyslogConnector {
-    fn write(&mut self, message: &[u8]) -> IoResult<usize> {
-        // eprintln!(
-        //     "syslog: got message \"{}\" ",
-        //     String::from_utf8_lossy(message)
-        // );
+#[derive(Debug)]
+enum SyslogConnector {
+    // Sends log lines to the syslog via a
+    // [UnixStream](https://doc.rust-lang.org/std/os/unix/net/struct.UnixStream.html).
+    #[cfg_attr(docsrs, doc(cfg(target_family = "unix")))]
+    #[cfg(target_family = "unix")]
+    Stream(std::os::unix::net::UnixStream),
 
+    // Sends log lines to the syslog via a
+    // [UnixDatagram](https://doc.rust-lang.org/std/os/unix/net/struct.UnixDatagram.html).
+    #[cfg_attr(docsrs, doc(cfg(target_family = "unix")))]
+    #[cfg(target_family = "unix")]
+    Datagram(std::os::unix::net::UnixDatagram),
+
+    // Sends log lines to the syslog via UDP.
+    //
+    // UDP is fragile and thus discouraged except for local communication.
+    Udp(UdpSocket),
+
+    // Sends log lines to the syslog via TCP.
+    Tcp(TcpStream),
+}
+
+impl Write for SyslogConnector {
+    fn write(&mut self, buf: &[u8]) -> IoResult<usize> {
         #[allow(clippy::match_same_arms)]
         match *self {
-            #[cfg(target_os = "linux")]
+            #[cfg(target_family = "unix")]
             Self::Datagram(ref ud) => {
                 // todo: reconnect if conn is broken
-                ud.send(message)
+                ud.send(buf)
             }
-            #[cfg(target_os = "linux")]
+            #[cfg(target_family = "unix")]
             Self::Stream(ref mut w) => {
                 // todo: reconnect if conn is broken
-                w.write(message)
-                    .and_then(|sz| w.write_all(&[0; 1]).map(|_| sz))
+                w.write(buf).and_then(|sz| w.write_all(&[0; 1]).map(|_| sz))
             }
             Self::Tcp(ref mut w) => {
                 // todo: reconnect if conn is broken
-                w.write(message)
+                w.write(buf)
             }
             Self::Udp(ref socket) => {
                 // ??
-                socket.send(message)
+                socket.send(buf)
             }
         }
     }
@@ -314,10 +340,10 @@ impl Write for SyslogConnector {
     #[allow(clippy::match_same_arms)]
     fn flush(&mut self) -> IoResult<()> {
         match *self {
-            #[cfg(target_os = "linux")]
+            #[cfg(target_family = "unix")]
             Self::Datagram(_) => Ok(()),
 
-            #[cfg(target_os = "linux")]
+            #[cfg(target_family = "unix")]
             Self::Stream(ref mut w) => w.flush(),
 
             Self::Udp(_) => Ok(()),
