@@ -5,10 +5,9 @@ mod d {
     use flate2::bufread::GzDecoder;
     use flexi_logger::{
         Cleanup, Criterion, DeferredNow, Duplicate, FileSpec, LogSpecification, Logger, Naming,
-        Record, WriteMode,
+        Record, WriteMode, TS_DASHES_BLANK_COLONS_DOT_BLANK,
     };
     use glob::glob;
-    use lazy_static::lazy_static;
     use log::*;
     use std::collections::BTreeMap;
     use std::fs::File;
@@ -16,10 +15,7 @@ mod d {
     use std::ops::Add;
     use std::path::{Path, PathBuf};
     use std::thread::{self, JoinHandle};
-    use time::{
-        format_description::{self, FormatItem},
-        OffsetDateTime,
-    };
+    use time::{format_description::FormatItem, macros::format_description, OffsetDateTime};
 
     const NO_OF_THREADS: usize = 5;
     const NO_OF_LOGLINES_PER_THREAD: usize = 20_000;
@@ -32,7 +28,7 @@ mod d {
         // we use a special log line format that starts with a special string so that it is easier to
         // verify that all log lines are written correctly
 
-        let start = super::test_utils::now_local_or_utc();
+        let start = super::test_utils::now_local();
         let directory = super::test_utils::dir();
         let logger = Logger::try_with_str("debug")
             .unwrap()
@@ -61,15 +57,15 @@ mod d {
 
         let worker_handles = start_worker_threads(NO_OF_THREADS);
         let new_spec = LogSpecification::parse("trace").unwrap();
-        thread::sleep(std::time::Duration::from_millis(1000));
+        thread::sleep(std::time::Duration::from_millis(500));
         logger.set_new_spec(new_spec);
 
         wait_for_workers_to_close(worker_handles);
 
-        let delta1 = (super::test_utils::now_local_or_utc() - start).whole_milliseconds();
+        let delta1 = (super::test_utils::now_local() - start).whole_milliseconds();
 
         std::mem::drop(logger);
-        let delta2 = (super::test_utils::now_local_or_utc() - start).whole_milliseconds();
+        let delta2 = (super::test_utils::now_local() - start).whole_milliseconds();
         println!(
             "Task executed with {} threads in {} ms, writing logs extended to {} ms.",
             NO_OF_THREADS, delta1, delta2
@@ -106,6 +102,7 @@ mod d {
             }
             debug!("({})  writing out line number {}", thread_number, idx);
         }
+        std::thread::sleep(std::time::Duration::from_millis(500));
         trace!("MUST_BE_PRINTED");
     }
 
@@ -118,12 +115,6 @@ mod d {
         trace!("All worker threads joined.");
     }
 
-    const TS_S: &str = "[year]-[month]-[day] [hour]:[minute]:[second].[subsecond digits:6] \
-                    [offset_hour sign:mandatory]:[offset_minute]";
-    lazy_static! {
-        static ref TS: Vec<FormatItem<'static>> = format_description::parse(TS_S).unwrap(/*ok*/);
-    }
-
     pub fn test_format(
         w: &mut dyn std::io::Write,
         now: &mut DeferredNow,
@@ -132,7 +123,7 @@ mod d {
         write!(
             w,
             "XXXXX [{}] T[{:?}] {} [{}:{}] {}",
-            now.now().format(&TS).unwrap(),
+            now.format(TS_DASHES_BLANK_COLONS_DOT_BLANK),
             thread::current().name().unwrap_or("<unnamed>"),
             record.level(),
             record.file().unwrap_or("<unnamed>"),
@@ -199,6 +190,10 @@ mod d {
             Box::new(BufReader::new(File::open(p).unwrap()))
         };
 
+        const TS: &[FormatItem<'static>] = format_description!(
+            "[year]-[month]-[day] [hour]:[minute]:[second].[subsecond digits:6] \
+                [offset_hour sign:mandatory]:[offset_minute]"
+        );
         for line in buf_reader.lines() {
             let line = line.unwrap();
             //9 fraction digits, should be 6
